package sun.font;

import java.text.Bidi;

public final class BidiUtils
{
    static final char NUMLEVELS = '>';
    
    public BidiUtils() {
    }
    
    public static void getLevels(Bidi bidi, byte[] array, int i) {
        int j = i + bidi.getLength();
        if (i < 0 || j > array.length) {
            throw new IndexOutOfBoundsException("levels.length = " + array.length + " start: " + i + " limit: " + j);
        }
        int runCount = bidi.getRunCount();
        for (int k = 0; k < runCount; ++k) {
            for (int n = i + bidi.getRunLimit(k), n2 = (byte)bidi.getRunLevel(k); i < n; array[i++] = n2) {}
        }
        return;
    }
    
    public static byte[] getLevels(Bidi bidi) {
        byte[] array = new byte[bidi.getLength()];
        getLevels(bidi, array, 0);
        return array;
    }
    
    public static int[] createVisualToLogicalMap(byte[] array) {
        int length = array.length;
        int[] array2 = new int[length];
        int n = 63;
        int i = 0;
        for (int j = 0; j < length; ++j) {
            array2[j] = j;
            int n2 = array[j];
            if (n2 > i) {
                i = n2;
            }
            if ((n2 & 0x1) != 0x0 && n2 < n) {
                n = n2;
            }
        }
        while (i >= n) {
            int n3 = 0;
            while (true) {
                if (n3 < length && array[n3] < i) {
                    ++n3;
                }
                else {
                    int k = n3++;
                    if (k == array.length) {
                        break;
                    }
                    while (n3 < length && array[n3] >= i) {
                        ++n3;
                    }
                    for (int n4 = n3 - 1; k < n4; ++k, --n4) {
                        int n5 = array2[k];
                        array2[k] = array2[n4];
                        array2[n4] = n5;
                    }
                }
            }
            i = (byte)(i - 1);
        }
        return array2;
    }
    
    public static int[] createInverseMap(int[] array) {
        if (array == null) {
            return null;
        }
        int[] array2 = new int[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[array[i]] = i;
        }
        return array2;
    }
    
    public static int[] createContiguousOrder(int[] array) {
        if (array != null) {
            return computeContiguousOrder(array, 0, array.length);
        }
        return null;
    }
    
    private static int[] computeContiguousOrder(int[] array, int n, int n2) {
        int[] array2 = new int[n2 - n];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = i + n;
        }
        int n4;
        for (int j = 0; j < array2.length - 1; n4 = array2[j], , array2[j] = n4, ++j) {
            int n3 = array[array2[j]];
            for (; j < array2.length; ++j) {
                if (array[array2[j]] < n3) {
                    n3 = array[array2[j]];
                }
            }
        }
        if (n != 0) {
            for (int k = 0; k < array2.length; ++k) {
                int n5 = k;
                array2[n5] -= n;
            }
        }
        int n6;
        for (n6 = 0; n6 < array2.length && array2[n6] == n6; ++n6) {}
        if (n6 == array2.length) {
            return null;
        }
        return createInverseMap(array2);
    }
    
    public static int[] createNormalizedMap(int[] array, byte[] array2, int n, int n2) {
        if (array != null) {
            if (n != 0 || n2 != array.length) {
                int n3;
                int n4;
                int n5;
                if (array2 == null) {
                    n4 = 0;
                    n3 = 1;
                    n5 = 1;
                }
                else if (array2[n] == array2[n2 - 1]) {
                    n4 = array2[n];
                    n5 = ((n4 & 0x1) == 0x0);
                    for (; n < n2 && array2[n] >= n4; ++n) {
                        if (n5 != 0) {
                            n5 = (array2[n] == n4);
                        }
                    }
                    n3 = (n == n2);
                }
                else {
                    n3 = 0;
                    n4 = 0;
                    n5 = 0;
                }
                if (n3 != 0) {
                    if (n5 != 0) {
                        return null;
                    }
                    int[] array3 = new int[n2 - n];
                    int n6;
                    if ((n4 & 0x1) != 0x0) {
                        n6 = array[n2 - 1];
                    }
                    else {
                        n6 = array[n];
                    }
                    if (n6 == 0) {
                        System.arraycopy(array, n, array3, 0, n2 - n);
                        return array3;
                    }
                    for (int i = 0; i < array3.length; ++i) {
                        array3[i] = array[i + n] - n6;
                    }
                    return array3;
                }
                else {
                    return computeContiguousOrder(array, n, n2);
                }
            }
            else {
                return array;
            }
        }
        else {
            return null;
        }
    }
    
    public static void reorderVisually(byte[] array, Object[] array2) {
        int length = array.length;
        int n = 63;
        int i = 0;
        for (int n2 : array) {
            if (n2 > i) {
                i = n2;
            }
            if ((n2 & 0x1) != 0x0 && n2 < n) {
                n = n2;
            }
        }
        while (i >= n) {
            int n3 = 0;
            while (true) {
                if (n3 < length && array[n3] < i) {
                    ++n3;
                }
                else {
                    int k = n3++;
                    if (k == array.length) {
                        break;
                    }
                    while (n3 < length && array[n3] >= i) {
                        ++n3;
                    }
                    for (int n4 = n3 - 1; k < n4; ++k, --n4) {
                        Object o = array2[k];
                        array2[k] = array2[n4];
                        array2[n4] = o;
                    }
                }
            }
            i = (byte)(i - 1);
        }
    }
}
